/*
  Source: http://stackoverflow.com/questions/16492736/what-is-the-this-pointer
  Purpose: example that illustrates how the pointer this works

  How 'this' works: 
  
  a) [this] refers to the current object.

      The keyword [this] identifies a special type of pointer. Suppose that you create 
      an object named x of class A, and class A has a non-static member function f().
      If you call the function x.f(), the keyword [this] in the body of f() stores the address of x.

  b) "[this] is a pointer that points to the object for which [this] function was called.
     For example, the function call A.max() will set the pointer this to the address of the object.
     The pointer this is acts as an implicit argument to all the member functions."

*/

#include <iostream>
using namespace std;

class Foo {
    public:
      Foo *foo() { return this; }
      const Foo *cfoo () const { return this; /* return foo(); is an error */ }
};

Foo x;       // can call either x.foo() or x.cfoo()
const Foo y; // can only call x.cfoo()

class Bar {
    int x;
    int y;
  public:
    // ο δημιουργός δίνει τιμές x = 1, y = 2.
    //Bar () : x(1), y(2) {}
    Bar(){
      x = 1;
      y = 2;
    }

    // η μέθοδος void bar(int x = 3) δίνει default τιμή στο y και στο όρισμα x
    void bar (int x = 3) {
        int y = 4; 
        std::cout << "x: " << x << std::endl;
        std::cout << "this->x: " << this->x << std::endl;
        std::cout << "y: " << y << std::endl;
        std::cout << "this->y: " << this->y << std::endl;
    }
};

int main(){
  cout << "\n\n"
       << "Δημιουργώ ένα αντικείμενο τύπου Bar με όνομα b:\n"
       << "Bar b;\n";         
  Bar b;
  
  cout <<"\n\n"
       << "Καλώ την συνάρτηση bar() της κλάσης Bar να δράσει στο αντικείμενο b:\n"
       << "b.bar();\n\n"
       << "-------------\n"
       << "Αποτελέσματα:\n"
       << "-------------\n";
  b.bar();

  cout << "\n-------------\n"
       << "Συμπεράσματα: \n"
       << "-------------\n"
       << "1a. Ο δείκτης this δείχνει στο τρέχον αντικείμενο.\n"
       << "1b. Δηλαδή στο αντικείμενο στο οποίο δρα η συνάρτηση που καλώ.\n"
       << "1c. Δηλαδή στο αντικείμενο αριστερά της τελείας.\n\n\n";

  return 0;
}