/*
friendFunctions.cpp

In principle, private and protected members of a class cannot be accessed from outside the same class in which
they are declared. However, this rule does not affect friends.

Friends are functions or classes declared as such.

If we want to declare an external function as friend of a class, thus allowing this function to have access to the
private and protected members of this class, we do it by declaring a prototype of this external function within the
class, and preceding it with the keyword friend.      
*/

#include <iostream>
using namespace std;

class CRectangle {
      int width, height;
      public:
             void set_values (int, int);
             int area () 
             {
                 return (width * height);
             }
             
             friend CRectangle duplicate (CRectangle);
};

void CRectangle::set_values (int a, int b) 
{
     width = a;
     height = b;
}

// returns a dupl
CRectangle duplicate (CRectangle rectparam)
{
    CRectangle rectres;
    rectres.width = rectparam.width;
    rectres.height = rectparam.height;
    
    return (rectres);
}

int main () {
    CRectangle rect, rectb;
    
    rect.set_values (2,3);
    rectb = duplicate (rect);
    cout << "area = " << rectb.area() << "\n";
     cout << "area b= " << rect.area() << "\n";
    return (0);       
}


/*
The duplicate function is a friend of CRectangle. From within that function we have been able to access the
members width and height of different objects of type CRectangle, which are private members. Notice that
neither in the declaration of duplicate() nor in its later use in main() have we considered duplicate a member
of class CRectangle. It isn't! It simply has access to its private and protected members without being a member.

The friend functions can serve, for example, to conduct operations between two different classes. Generally, the
use of friend functions is out of an object-oriented programming methodology, so whenever possible it is better to
use members of the same class to perform operations with them. Such as in the previous example, it would have
been shorter to integrate duplicate() within the class CRectangle.
*/
